@@ -22,20 +22,22 @@
 # Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 #
 
+import collections
 import gzip
 import html
 import importlib
 import os
+import os.path
 import re
 import shutil
 import sqlite3
 import subprocess
 import sys
-import urllib.request
 
 from bs4 import BeautifulSoup
 from cppman import environ, util
 from cppman.crawler import Crawler
+from urllib.parse import urlparse, unquote
 
 
 
@@ -95,6 +97,16 @@ def _commonprefix(s1, s2):
             return s1[:i]
     return s1
 
+def _removeprefix(string, prefix):
+    if prefix and string.startswith(prefix):
+        return string[len(prefix):]
+    return string
+
+def _removesuffix(string, suffix):
+    if suffix and string.endswith(suffix):
+        return string[:-len(suffix)]
+    return string
+
 class Cppman(Crawler):
     """ Manage cpp man pages, indexes. """
 
@@ -114,7 +126,7 @@ class Cppman(Crawler):
             self.add_url_filter('\.(jpg|jpeg|gif|png|js|css|swf|svg)$')
             self.set_follow_mode(Crawler.F_SAME_PATH)
 
-            sources = [('cplusplus.com', 'http://www.cplusplus.com/reference/', None),
+            sources = [('cplusplus.com', 'https://cplusplus.com/reference/', None),
                        ('cppreference.com', 'https://en.cppreference.com/w/cpp', '/w/cpp')]
 
             for table, url, path in sources:
@@ -142,7 +154,9 @@ class Cppman(Crawler):
                     ')' % (table, table))
 
                 """ Crawl and insert all entries. """
-                results = self.crawl(url)
+                self.results = collections.defaultdict(list)
+                self.crawl(url)
+                results = self._results_with_unique_title()
 
                 for title in results:
                     """ 1. insert title """
@@ -232,15 +246,16 @@ class Cppman(Crawler):
         name = self._extract_name(content).replace('\n', '')
         keywords = self._extract_keywords(content)
 
-        self.results[name] = {'url': url, 'keywords': set(), 'aliases': set()}
+        entry = {'url': url, 'keywords': set(), 'aliases': set()}
+        self.results[name].append(entry)
 
         for n in self._parse_title(name):
             """ add as keyword """
-            self.results[name]["keywords"].add(n)
+            entry["keywords"].add(n)
 
             """ add as keyword without std:: """
             if n.find("std::") != -1:
-                self.results[name]["keywords"].add(n.replace('std::', ''))
+                entry["keywords"].add(n.replace('std::', ''))
 
             """ add with all keywords variations """
             for k in keywords:
@@ -248,24 +263,48 @@ class Cppman(Crawler):
                 if n.find("std::") != -1 and k.find("std::") == -1:
                     k = "std::" + k;
 
-                self.results[name]["aliases"].add((n, k))
+                entry["aliases"].add((n, k))
                 prefix = _commonprefix(n, k)
 
                 if len(prefix) > 2 and prefix[-2:] == "::":
                     """ Create names and keyword without prefixes """
                     new_name = n[len(prefix):]
                     new_key  = k[len(prefix):]
-                    self.results[name]["aliases"].add((new_name, new_key))
+                    entry["aliases"].add((new_name, new_key))
 
                 if k.find("std::") != -1:
-                    self.results[name]["aliases"].add(
+                    entry["aliases"].add(
                         (n, k.replace('std::', '')))
 
         return True
 
+    def _results_with_unique_title(self):
+        """process crawling results and return title -> entry dictionary;
+           add part of the path to entries having the same title
+        """
+        results = dict()
+        for title, entries in self.results.items():
+            if len(entries) == 1:
+                results[title] = entries[0]
+            else:
+                paths = [_removesuffix(urlparse(entry['url'])[2], '/') for entry in entries]
+                prefix = os.path.commonpath(paths)
+                if prefix:
+                    prefix += '/'
+                suffix = '/' + os.path.basename(paths[0])
+                for path in paths:
+                    if not path.endswith(suffix):
+                        suffix = ''
+                        break
+                for index, entry in enumerate(entries):
+                    path = _removeprefix(paths[index], prefix)
+                    path = _removesuffix(path, suffix)
+                    results["{} ({})".format(title, unquote(path))] = entry
+        return results
+
     def _extract_name(self, data):
         """Extract man page name from web page."""
-        name = re.search('<h1[^>]*>(.+?)</h1>', data).group(1)
+        name = re.search('<[hH]1[^>]*>(.+?)</[hH]1>', data, re.DOTALL).group(1)
         name = re.sub(r'<([^>]+)>', r'', name)
         name = re.sub(r'&gt;', r'>', name)
         name = re.sub(r'&lt;', r'<', name)
@@ -358,7 +397,8 @@ class Cppman(Crawler):
                         typedefTable = True
                     elif typedefTable:
                         res = re.search('^\s*(\S*)\s+.*$', tds[0].get_text())
-                        names.append(res.group(1))
+                        if res and res.group(1):
+                            names.append(res.group(1))
                     elif not typedefTable:
                         break
             if typedefTable:
@@ -458,7 +498,7 @@ class Cppman(Crawler):
 
         # There are often some errors in the HTML, for example: missing closing
         # tag. We use fixupHTML to fix this.
-        data = util.fixupHTML(urllib.request.urlopen(url).read())
+        data = util.fixupHTML(util.urlopen(url).read())
 
         formatter = importlib.import_module(
             'cppman.formatter.%s' % source[:-4])
